#!/bin/bash

# Nom de la base de donnes
dbname=todolist
# propritaire du processus http.
owner=djedi


createdb $dbname
psql $dbname -e <<EOF
begin;
---- Les utilisateurs de la TODO List -----
-- Login
create table users (
  id          serial primary key,   -- id unique pour tous les users
  login       varchar(16),          -- le nom des utilisateurs
  passwd      char(13),             -- le mot de passe
  properties  int                   -- les permissions de l'utilisateurs
);

---- Les annotations -----
create table annotations (
  id          serial primary key,  -- cl primaire
  state       int,                 -- status de l'annotation
  parent      int,                 -- id objet parent (voir type)
  type        int,                 -- type de l'objet parent (voir parent)
  date        timestamp,           -- date de cration
  text        text                 -- texte de l'annotation...
);


---- Les rpertoires -----
create table directories (
 id           serial primary key,             -- cle primaire
 name         varchar(256),                   -- le nom du repertoire
 state        int,                            -- bit-field pour extension 
 parent       int references directories(id), -- repertoire parent
 owner        int references users(id),       -- propriaitaire du repertoire
 annotation   int references annotations(id)  -- Commentaire du repertoire
);

----- Les notes -----
create table cards (
  id          serial primary key,              -- cl primaire
  title       varchar(256),                    -- titre de la note
  state       int,                             -- status de la note (pour extensions)
  parent      int references directories(id),  -- rpertoire parent
  owner       int references users(id),        -- propriaitaire de la note
  annotation  int references annotations(id),  -- Commentaire du repertoire  
  expiration  timestamp,                       -- date d'expiration
  destruction timestamp                        -- date de destruction
);

-------------------------
-- Droits au 
grant select,update,insert on users, users_id_seq to "$owner";
grant select,update,insert on directories, directories_id_seq to "$owner";
grant select,update,insert on cards, cards_id_seq to "$owner";
grant select,update,insert on annotations, annotations_id_seq to "$owner";
commit;

begin;
insert into users(login,passwd) values ('djedi','titi');

insert into directories(name,parent,owner,state) values ('root',1,1,0); 
insert into directories(name,parent,owner,state) values ('a',1,1,0); 
insert into directories(name,parent,owner,state) values ('b',1,1,0); 
insert into directories(name,parent,owner,state) values ('c',2,1,0); 
insert into cards(title,parent,owner,state) values ('un titre',2,1,0); 
commit;
EOF