//   DatabaseAcces.java --- TodoList

//   Copyright (C) 2003 Moise Valvassori <djedi@ai.univ-paris8.fr>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.todolist;
import java.util.Properties;
import java.util.Date;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.sql.SQLException;
import java.sql.DriverManager;

/**
 * Un acces a la base.
 * Une fois logg, toutes les requtes sont identifi!!!
 * @author "Moise Valvassori" <djedi@ai.univ-paris8.fr>
 * @version 1.0
 * @since Mon Feb  3 17:01:23 2003
 */
public class DatabaseAccess {
  private Connection con;
  private Properties prop;
  private int uid = -1;


  private void debug (String d) {
    //    System.out.println("DB: "+d);
  }

  /**
   * Creates a new <code>DataBaseAcces</code> instance.
   *
   */
  public DatabaseAccess (Properties p) {
    prop=p;
    try {
      Class.forName("org.postgresql.Driver");
    } catch (ClassNotFoundException e){
      System.err.println("PostGres Driver not found...");
      System.exit(1);
    }
  }


  private ResultSet query (String q) {
    try {
      debug(q);
      if (con == null)
	connect();
      Statement stm = con.createStatement();
      return stm.executeQuery(q);
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  private void update (String q) {
    try {
      debug(q);
      if (con == null)
	connect();
      Statement stm = con.createStatement();
      stm.executeUpdate(q);
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }

  public void connect() {
    // jdbc:postgresql://host/base
    try {
      con = DriverManager.getConnection("jdbc:postgresql://"+prop.getProperty("db.host")+"/"+prop.getProperty("db.name"),
					prop.getProperty("db.username"), prop.getProperty("db.passwd"));
    } catch (SQLException e) {
	System.err.println(e);
    }
  }

  public void close() {
    try {
      if (con != null)
	con.close();
    } catch (SQLException e) {
      e.printStackTrace();
    }
  }


  public boolean newUser(String login, String passwd) {
    try {
      debug(login);
      debug(passwd);

      ResultSet rs = query("select login,passwd from users;");
      if(rs.getFetchSize()!= 0)
	return false;

      update("insert into users(login,passwd) values ('"+login+"','"+passwd+"');");
      // Penser a cree un repertoire racine...
      rs.close();
    } catch (SQLException e) {
      e.printStackTrace();
    }
    return true;
  }

  public boolean login(String login, String passwd) {
    try {
      debug(login);
      debug(passwd);

      ResultSet rs = query("select id from users;");
      if(rs.getFetchSize()!= 1)
	return false;

      rs.next();
      uid = rs.getInt("id");
      debug(""+uid);
      rs.close();
    } catch (SQLException e) {
      e.printStackTrace();
    }
    return true;
  }


  /* Mthode lies aux TreeModel */

  public DirectoryNode getRootDir() {
    try {
      ResultSet rs = query("select id,name from directories where owner="+uid+" and parent = id;");

      rs.next();
      DirectoryNode node = new DirectoryNode(rs.getInt("id"),rs.getString("name"),0);
      rs.close();
      debug(""+node);
      return node;
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  public int getChildCount(DirectoryNode n) {
    try {
      ResultSet rs = query("select count(id) from directories where owner="+uid
			   + " and state & 1 = 0"
			   + " and parent = "+ n.getId()
			   + " and parent != id;");

      rs.next();
      int r = rs.getInt(1);
      rs.close();

      rs = query("select count(id) from cards where owner="+uid
        	 + " and state & 1 = 0"
		 + " and parent = "+ n.getId()
		 + " and parent != id;");
      rs.next();
      r += rs.getInt(1);
      rs.close();

      return r;
    } catch (SQLException e) {
      e.printStackTrace();
      return 0;
    }
  }

  public DirectoryNode getChild(DirectoryNode n, int c) {
    try {
      ResultSet rs = query("select id,name from directories where owner="+uid
          	           + " and state & 1 = 0"
			   + " and parent = "+ n.getId()
			   + " and parent != id limit 1 offset "+c+";");

      if (rs.getFetchSize()!=0) {
	rs.next();
	DirectoryNode node = new DirectoryNode(rs.getInt("id"),rs.getString("name"),0);
	rs.close();
	return node;
      } else {
	rs = query("select count(id) from directories where owner="+uid
        	   + " and state & 1 = 0"
		   + " and parent = "+ n.getId()
		   + " and parent != id;");

	rs.next();
	c -= rs.getInt(1);
	rs.close();
	rs = query("select id,title,expiration,destruction,annotation from cards where owner="+uid
        	   + " and state & 1 = 0"
		   + " and parent = "+ n.getId()
		   + " and parent != id limit 1 offset "+c+";");
	rs.next();
	DirectoryNode node = new DirectoryNode(rs.getInt("id"),rs.getString("title"),1);
	node.set(rs.getDate("expiration"), rs.getDate("destruction"));
	int i = rs.getInt("annotation");
	if (i != 0) {
	  rs.close();
	  rs = query("select text from annotations where id = "+i+";");
	  rs.next();
	  node.setAnnotation(rs.getString("text"));
	  node.setAnnotationId(i);
	}

	rs.close();
	return node;
      }
    } catch (SQLException e) {
      e.printStackTrace();
      return null;
    }
  }

  public int updateCard (int cardid, String title, Date expire, Date destroy, String ann,int aid) {
    update("begin;");
    String q = "update cards set title='"+title+"'";
    if (expire != null)
      q += ", expiration='"+expire+"'";
    else
      q += ", expiration='Thu Jan 01 01:00:00 CET 1970'";

    if (destroy != null)
      q += ", destruction='"+destroy+"'";
    else
      q += ", destruction='Thu Jan 01 01:00:00 CET 1970'";

    update(q+" where id="+cardid+";");

    if (ann != null) {
      if (aid != 0)
	update ("update annotations set text = '"+ann+"' where id = "+aid+";");
      else {
	update ("insert into annotations(text) values('"+ann+"');");
	try {
	  ResultSet rs = query("select id from annotations where text='"+ann+"';");
	  rs.next();
	  aid = rs.getInt("id");
	  rs.close();
	  update ("update cards set annotation = '"+aid+"' where id = "+cardid+";");
	} catch (SQLException e) {
	  e.printStackTrace();
	} // end of try-catch
      }
    }

    update("commit;");
    return aid;
  }

  public void updateDir (int cardid, String title) {
    String q = "update directories set name='"+title+"'";
    update(q+" where id="+cardid+";");
  }


  public int addDirectory(DirectoryNode p,String name) {
    if (p == null)
      return 0;

    update("insert into directories(name,parent,owner,state) values ('"+name+"',"+p.getId()+","+uid+",0);");
    try {
      ResultSet rs = query("select id from directories where owner="+uid+
			   " and parent="+p.getId()+" and name = '"+name+"';");
      rs.next();
      return rs.getInt(1);
    } catch (SQLException e) {
      e.printStackTrace();
      return 0;
    }
  }

  public int addCard(DirectoryNode p,String name) {
    if (p == null)
      return 0;

    update("insert into cards(title,parent,owner,state) values ('"+name+"',"+p.getId()+","+uid+",0);");
    try {
      ResultSet rs = query("select id from cards where owner="+uid+
			   " and parent="+p.getId()+" and title = '"+name+"';");
      rs.next();
      return rs.getInt(1);
    } catch (SQLException e) {
      e.printStackTrace();
      return 0;
    }
  }

  public void remove(DirectoryNode p) {
    if (p == null)
      return;

    String table;
    if (p.getType() == 0)
      table = "directories";
    else
      table = "cards";

    update("update "+table+" set state = 1 where id = "+p.getId()+";");
  }
}
