//   DirectoryDBTreeModel.java --- Todo List

//   Copyright (C) 2003 Valvassori Mose <moise@valvassori.org>

//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.

//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.

//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the Free Software
//   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

package org.djedi.todolist.gui;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.Properties;
import javax.swing.event.TreeModelEvent;
import javax.swing.event.TreeModelListener;
import javax.swing.tree.TreeModel;
import javax.swing.tree.TreePath;
import org.djedi.todolist.DatabaseAccess;
import org.djedi.todolist.DirectoryNode;

/**
 * TreeModel des repertoire de la Base de Donnes.
 * @author "Valvassori Mose" <moise@valvassori.org>
 * @version $Id: DirectoryDBTreeModel.java,v 1.12 2003/02/11 16:43:38 djedi Exp $
 * @since Tue Feb  4 09:35:20 2003
 */
public class DirectoryDBTreeModel implements TreeModel  {
  private DatabaseAccess dba;
  private Properties prop;
  private DirectoryNode root = null;
  private ArrayList treemodellistener;

  /**
   * Creates a new <code>DirectoryDBTreeModel</code> instance.
   *
   */
  public DirectoryDBTreeModel (DatabaseAccess dba,Properties p) {
    prop = p;
    this.dba = dba;
    treemodellistener = new ArrayList();
  }


  public Object getChild(Object param1, int param2)
  {
    DirectoryNode n = (DirectoryNode)((DirectoryNode)param1).getChild(param2);
    if (n == null) {
      DirectoryNode i = dba.getChild((DirectoryNode)param1, param2);
      ((DirectoryNode)param1).setChild(i, param2);
      return i;
    }
    
    return n;
  }

  public Object getRoot()
  {
    if (root == null)
      root = dba.getRootDir();
    return root;
  }

  public int getChildCount(Object param1)
  {
    DirectoryNode n = (DirectoryNode)param1;
    if (n.getChildCount()==-1)
      n.setChildCount(dba.getChildCount(n));
    return n.getChildCount();
  }

  public boolean isLeaf(Object param1)
  {
    DirectoryNode n = ((DirectoryNode)param1);
    return (n.getType()!=0) ;
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   */
  public void valueForPathChanged(TreePath path, Object param2)
  {
    // TODO: implement this javax.swing.tree.TreeModel method
    for (Iterator i = treemodellistener.iterator(); i.hasNext();){
      TreeModelListener m = (TreeModelListener)i.next();
      m.treeNodesChanged(new TreeModelEvent(this,path));
    }
  }

  /**
   *
   * @param param1 <description>
   * @param param2 <description>
   * @return <description>
   */
  public int getIndexOfChild(Object param1, Object param2)
  {
    // TODO: implement this javax.swing.tree.TreeModel method
    return 0;
  }

  /**
   *
   * @param param1 <description>
   */
  public void addTreeModelListener(TreeModelListener param1)
  {
    treemodellistener.add(param1);
  }

  /**
   *
   * @param param1 <description>
   */
  public void removeTreeModelListener(TreeModelListener param1)
  {
    // TODO: implement this javax.swing.tree.TreeModel method
  }

  public void addDirectory(DirectoryNode p,TreePath tp,String name) {
    int id = dba.addDirectory(p,name);
    int n = p.addChild(name,id,0);

    int [] inds = {n};
    Object [] srcs = {p.getChild(n)};

    TreeModelEvent tme = new TreeModelEvent (p,tp,inds,srcs);
    for (Iterator i=treemodellistener.iterator(); i.hasNext();) {
      TreeModelListener tml = (TreeModelListener)i.next();
      tml.treeNodesInserted(tme);
    }
  }

  public void addCard(DirectoryNode p,TreePath tp,String name) {
    int id = dba.addCard(p,name);
    int n = p.addChild(name,id,1);

    int [] inds = {n};
    Object [] srcs = {p.getChild(n)};

    TreeModelEvent tme = new TreeModelEvent (p,tp,inds,srcs);
    for (Iterator i=treemodellistener.iterator(); i.hasNext();) {
      TreeModelListener tml = (TreeModelListener)i.next();
      tml.treeNodesInserted(tme);
    }
  }

  public void remove(DirectoryNode p,TreePath tp) {
    dba.remove(p);
    int n = ((DirectoryNode)tp.getParentPath().getLastPathComponent()).remove(p);

    int [] inds = {n};
    Object [] srcs = {tp};

    TreeModelEvent tme = new TreeModelEvent (this,tp.getParentPath(),inds,srcs);
    for (Iterator i=treemodellistener.iterator(); i.hasNext();) {
      TreeModelListener tml = (TreeModelListener)i.next();
      tml.treeNodesRemoved(tme);
    }
  }
}
