/// MainFrame.java --- Todo List

/// Copyright (C) 2003 Valvassori Mose <moise@valvassori.org>

/// This program is free software; you can redistribute it and/or modify
/// it under the terms of the GNU General Public License as published by
/// the Free Software Foundation; either version 2 of the License, or
/// (at your option) any later version.

/// This program is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
/// GNU General Public License for more details.

/// You should have received a copy of the GNU General Public License
/// along with this program; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
package org.djedi.todolist.gui;

import javax.swing.JFrame;
import javax.swing.tree.TreePath;
import org.djedi.todolist.DatabaseAccess;
import org.djedi.todolist.DirectoryNode;
import java.util.Properties;
import java.awt.CardLayout;
import javax.swing.tree.TreeSelectionModel;
import javax.swing.event.TreeSelectionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.JPopupMenu;
import javax.swing.Action;
import javax.swing.UIManager;
import java.util.Locale;
import java.awt.event.KeyEvent;

/**
 *
 * @author "Valvassori Mose" <moise@valvassori.org>
 * @version $Id: MainFrame.java,v 1.20 2003/02/14 16:03:14 djedi Exp $
 * @since Tue Feb  4 09:34:25 2003
 * @see JFrame
 */
public class MainFrame extends JFrame {
  private DatabaseAccess dba;
  private Properties prop;
  private TreePath currentpath;
  private DirectoryNode currentNode;
  private JPopupMenu treePopup=null;
  private Action addCardAction;
  private Action addDirAction;
  private Action removeAction;
  private Action cutAction;
  private Action copyAction;
  private Action pasteAction;
  private DirectoryDBTreeModel dtmodel;

    /** Creates new form MainFrame */
    public MainFrame(DatabaseAccess dba,Properties p) {
      prop = p;
      this.dba = dba;

      if (p.getProperty("gui.lookfeel") != null) try {
	UIManager.setLookAndFeel(p.getProperty("gui.lookfeel"));
      } catch (Exception e) {
      }

      if (p.getProperty("gui.locale") != null) try {
	String l = p.getProperty("gui.locale");
	Locale.setDefault(new Locale(l.substring(0,2),l.substring(3,5)));
      } catch (Exception e) {
      }

      addCardAction = new AddAction(this);
      addCardAction.setEnabled(false);
      addDirAction = new AddDirAction(this);
      addDirAction.setEnabled(false);
      removeAction = new RemoveAction(this);
      removeAction.setEnabled(false);
      cutAction = new CutAction(this);
      cutAction.setEnabled(false);
      copyAction = new CopyAction(this);
      copyAction.setEnabled(false);
      pasteAction = new PasteAction(this);
      pasteAction.setEnabled(false);

      initComponents();
      directoryTree.setEnabled(false);
      directoryTree.setModel(null);
      directoryTree.getSelectionModel().setSelectionMode
	(TreeSelectionModel.SINGLE_TREE_SELECTION);

      directoryTree.addTreeSelectionListener(new TreeSelectionListener() {
	  public void valueChanged(TreeSelectionEvent e) {
	    DirectoryNode node = (DirectoryNode)directoryTree.getLastSelectedPathComponent();
	    if (node == null) return;
	    selectNode(node);
	  }
	});
      directoryTree.putClientProperty("JTree.lineStyle", "Angled");

    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        jSplitPane1 = new javax.swing.JSplitPane();
        jPanel7 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        directoryTree = new javax.swing.JTree();
        jToolBar1 = new javax.swing.JToolBar();
        bacard = new javax.swing.JButton();
        badir = new javax.swing.JButton();
        bdel = new javax.swing.JButton();
        bcut = new javax.swing.JButton();
        bcopy = new javax.swing.JButton();
        bpaste = new javax.swing.JButton();
        container = new javax.swing.JPanel();
        empty = new javax.swing.JPanel();
        note = new javax.swing.JPanel();
        jPanel2 = new javax.swing.JPanel();
        jLabel1 = new javax.swing.JLabel();
        cardTitle = new javax.swing.JTextField();
        jPanel3 = new javax.swing.JPanel();
        ccompleted = new javax.swing.JCheckBox();
        jPanel1 = new javax.swing.JPanel();
        jLabel2 = new javax.swing.JLabel();
        cardCdate = new javax.swing.JLabel();
        jPanel4 = new javax.swing.JPanel();
        jLabel4 = new javax.swing.JLabel();
        cardEdate = new kiwi.ui.DateChooserField();
        jPanel5 = new javax.swing.JPanel();
        jLabel6 = new javax.swing.JLabel();
        cardDdate = new kiwi.ui.DateChooserField();
        jScrollPane3 = new javax.swing.JScrollPane();
        cardText = new javax.swing.JTextArea();
        jPanel6 = new javax.swing.JPanel();
        jButton3 = new javax.swing.JButton();
        directory = new javax.swing.JPanel();
        jPanel8 = new javax.swing.JPanel();
        directoryTitle = new javax.swing.JLabel();
        dirName = new javax.swing.JTextField();
        jScrollPane2 = new javax.swing.JScrollPane();
        dirAnnotation = new javax.swing.JEditorPane();
        jPanel9 = new javax.swing.JPanel();
        jButton4 = new javax.swing.JButton();
        jMenuBar1 = new javax.swing.JMenuBar();
        jMenu1 = new javax.swing.JMenu();
        loginItem = new javax.swing.JMenuItem();
        logoutItem = new javax.swing.JMenuItem();
        jSeparator1 = new javax.swing.JSeparator();
        quitItem = new javax.swing.JMenuItem();

        setTitle(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Todo_List"));
        addWindowListener(new java.awt.event.WindowAdapter() {
            public void windowClosing(java.awt.event.WindowEvent evt) {
                exitForm(evt);
            }
        });

        jSplitPane1.setEnabled(false);
        jPanel7.setLayout(new java.awt.BorderLayout());

        jScrollPane1.setPreferredSize(new java.awt.Dimension(100, 200));
        directoryTree.setEditable(true);
        directoryTree.setMinimumSize(new java.awt.Dimension(100, 50));
        directoryTree.addMouseListener(new java.awt.event.MouseAdapter() {
            public void mousePressed(java.awt.event.MouseEvent evt) {
                rightClickEvent(evt);
            }
        });

        jScrollPane1.setViewportView(directoryTree);

        jPanel7.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        // ceci est ici
        bacard.setAction(addCardAction);
        bacard.setText(null);
        jToolBar1.add(bacard);

        badir.setAction(addDirAction);
        badir.setText(null);
        jToolBar1.add(badir);

        bdel.setAction(removeAction);
        bdel.setText(null);
        jToolBar1.add(bdel);

        bcut.setAction(cutAction);
        bcut.setText(null);
        jToolBar1.add(bcut);

        bcopy.setAction(copyAction);
        bcopy.setText(null);
        jToolBar1.add(bcopy);

        bpaste.setAction(pasteAction);
        bpaste.setText(null);
        jToolBar1.add(bpaste);

        jPanel7.add(jToolBar1, java.awt.BorderLayout.NORTH);

        jSplitPane1.setLeftComponent(jPanel7);

        container.setLayout(new java.awt.CardLayout());

        container.add(empty, "empty");

        note.setLayout(new javax.swing.BoxLayout(note, javax.swing.BoxLayout.Y_AXIS));

        note.setBorder(new javax.swing.border.TitledBorder("Card"));
        jLabel1.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Title"));
        jPanel2.add(jLabel1);

        cardTitle.setColumns(17);
        jPanel2.add(cardTitle);

        note.add(jPanel2);

        jPanel3.setLayout(new javax.swing.BoxLayout(jPanel3, javax.swing.BoxLayout.Y_AXIS));

        jPanel3.setBorder(new javax.swing.border.EtchedBorder());
        jPanel3.setMinimumSize(new java.awt.Dimension(243, 3));
        ccompleted.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Completed"));
        jPanel3.add(ccompleted);

        jLabel2.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Creation_Date:"));
        jLabel2.setFont(new java.awt.Font("Dialog", 0, 12));
        jPanel1.add(jLabel2);

        cardCdate.setFont(new java.awt.Font("Dialog", 0, 12));
        jPanel1.add(cardCdate);

        jPanel3.add(jPanel1);

        jLabel4.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Expiration_Date:"));
        jLabel4.setFont(new java.awt.Font("Dialog", 0, 12));
        jPanel4.add(jLabel4);

        cardEdate.setPreferredSize(new java.awt.Dimension(100, 23));
        cardEdate.setFont(new java.awt.Font("Dialog", 0, 10));
        jPanel4.add(cardEdate);

        jPanel3.add(jPanel4);

        jLabel6.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Destruction_Date:"));
        jLabel6.setFont(new java.awt.Font("Dialog", 0, 12));
        jPanel5.add(jLabel6);

        cardDdate.setPreferredSize(new java.awt.Dimension(100, 23));
        cardDdate.setFont(new java.awt.Font("Dialog", 0, 10));
        jPanel5.add(cardDdate);

        jPanel3.add(jPanel5);

        note.add(jPanel3);

        jScrollPane3.setPreferredSize(new java.awt.Dimension(300, 250));
        cardText.setWrapStyleWord(true);
        jScrollPane3.setViewportView(cardText);

        note.add(jScrollPane3);

        jPanel6.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButton3.setIcon(new javax.swing.ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Export16.gif")));
        jButton3.setMnemonic(KeyEvent.VK_ENTER);
        jButton3.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Update"));
        jButton3.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateAction(evt);
            }
        });

        jPanel6.add(jButton3);

        note.add(jPanel6);

        container.add(note, "note");

        directory.setLayout(new javax.swing.BoxLayout(directory, javax.swing.BoxLayout.Y_AXIS));

        directory.setBorder(new javax.swing.border.TitledBorder(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("directory")));
        directoryTitle.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("name"));
        jPanel8.add(directoryTitle);

        dirName.setColumns(17);
        jPanel8.add(dirName);

        directory.add(jPanel8);

        jScrollPane2.setPreferredSize(new java.awt.Dimension(300, 250));
        jScrollPane2.setViewportView(dirAnnotation);

        directory.add(jScrollPane2);

        jPanel9.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.RIGHT));

        jButton4.setIcon(new javax.swing.ImageIcon(getClass().getResource("/toolbarButtonGraphics/general/Export16.gif")));
        jButton4.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Update"));
        jButton4.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                updateDirAction(evt);
            }
        });

        jPanel9.add(jButton4);

        directory.add(jPanel9);

        container.add(directory, "dir");

        jSplitPane1.setRightComponent(container);

        getContentPane().add(jSplitPane1, java.awt.BorderLayout.CENTER);

        jMenu1.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Todo"));
        loginItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_L, java.awt.event.InputEvent.CTRL_MASK));
        loginItem.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Login"));
        loginItem.setIcon(new javax.swing.ImageIcon(getClass().getResource("/toolbarButtonGraphics/navigation/Home16.gif")));
        loginItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                loginEvent(evt);
            }
        });

        jMenu1.add(loginItem);

        logoutItem.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Logout"));
        logoutItem.setEnabled(false);
        logoutItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                logoutItemActionPerformed(evt);
            }
        });

        jMenu1.add(logoutItem);

        jMenu1.add(jSeparator1);

        quitItem.setAccelerator(javax.swing.KeyStroke.getKeyStroke(java.awt.event.KeyEvent.VK_Q, java.awt.event.InputEvent.CTRL_MASK));
        quitItem.setText(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("Quit"));
        quitItem.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                quitEvent(evt);
            }
        });

        jMenu1.add(quitItem);

        jMenuBar1.add(jMenu1);

        setJMenuBar(jMenuBar1);

        pack();
    }//GEN-END:initComponents

    private void updateDirAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateDirAction
	// Add your handling code here:
      dba.updateDir(currentNode.getId(), dirName.getText());
      currentNode.setName(dirName.getText());
      directoryTree.getModel().valueForPathChanged(currentpath,null);
    }//GEN-LAST:event_updateDirAction

    private void updateAction(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_updateAction
	// Add your handling code here:
      int aid = dba.updateCard(currentNode.getId(), cardTitle.getText(),
			       cardEdate.getDate(), cardDdate.getDate(),
			       cardText.getText(),currentNode.getAnnotationId());
      currentNode.setName(cardTitle.getText());
      currentNode.set(cardEdate.getDate(), cardDdate.getDate());
      currentNode.setAnnotation(cardText.getText());
      currentNode.setAnnotationId(aid);
      directoryTree.getModel().valueForPathChanged(currentpath,null);
    }//GEN-LAST:event_updateAction

    private void rightClickEvent(java.awt.event.MouseEvent evt) {//GEN-FIRST:event_rightClickEvent
      // Add your handling code here:
      TreePath path = directoryTree.getPathForLocation(evt.getX(),evt.getY());
      if (evt.isPopupTrigger()) {
	if (path != null) {
	  directoryTree.setSelectionPath(path);
	  if (treePopup==null)
	    createPopup();
	  treePopup.show(directoryTree,evt.getX(),evt.getY());
	}
      }
    }//GEN-LAST:event_rightClickEvent


  private void createPopup () {
    treePopup = new JPopupMenu(java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("menu"));
    treePopup.add(addCardAction);
    treePopup.add(addDirAction);
    treePopup.addSeparator();
    treePopup.add(removeAction);
    treePopup.addSeparator();
    treePopup.add(cutAction);
    treePopup.add(copyAction);
    treePopup.add(pasteAction);
  }

  private void selectNode(DirectoryNode n) {
    CardLayout l = (CardLayout)container.getLayout();

    removeAction.setEnabled(true);
    switch (n.getType()) {
    case 0:
      l.show(container,java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("dir"));
      addCardAction.setEnabled(true);
      addDirAction.setEnabled(true);

      dirName.setText(n.toString());
      currentNode = n;
      currentpath = directoryTree.getSelectionPath();
      break;
    case 1:
      l.show(container,java.util.ResourceBundle.getBundle("org/djedi/todolist/gui/gui").getString("note"));
      currentpath = directoryTree.getSelectionPath();
      currentNode = n;
      cardTitle.setText(n.toString());
      cardEdate.setDate(n.getExpiration());
      cardDdate.setDate(n.getDestruction());
      cardText.setText(n.getAnnotation());

      addCardAction.setEnabled(false);
      addDirAction.setEnabled(false);
      break;
    }
  }

  private void loginEvent(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_loginEvent
	// Add your handling code here:
	new LoginDialog(this,true,prop.getProperty("user.login"),prop.getProperty("user.passwd")).show();
    }//GEN-LAST:event_loginEvent

    private void logoutItemActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_logoutItemActionPerformed
	// Add your handling code here:
    }//GEN-LAST:event_logoutItemActionPerformed

    private void quitEvent(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_quitEvent
	// Add your handling code here:
      dba.close();
      System.exit(0);
    }//GEN-LAST:event_quitEvent

    /** Exit the Application */
    private void exitForm(java.awt.event.WindowEvent evt) {//GEN-FIRST:event_exitForm
      dba.close();
      System.exit(0);
    }//GEN-LAST:event_exitForm


  public DirectoryNode getCurrentNode() {
    return currentNode;
  }
  public TreePath getCurrentTreePath() {
    return currentpath;
  }

  public DirectoryDBTreeModel getTreeModel () {
    return dtmodel;
  }

  public void logged () {
    jSplitPane1.setEnabled(true);
    directoryTree.setEnabled(true);
    logoutItem.setEnabled(true);
    loginItem.setEnabled(false);
    dtmodel = new DirectoryDBTreeModel(dba,prop);
    directoryTree.setModel(dtmodel);
  }
  public DatabaseAccess getDatabaseAccess() {
    return dba;
  }

    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JPanel directory;
    private javax.swing.JMenuItem loginItem;
    private javax.swing.JLabel cardCdate;
    private javax.swing.JToolBar jToolBar1;
    private javax.swing.JButton jButton4;
    private javax.swing.JButton jButton3;
    private javax.swing.JMenu jMenu1;
    private javax.swing.JTextField dirName;
    private javax.swing.JScrollPane jScrollPane3;
    private javax.swing.JScrollPane jScrollPane2;
    private javax.swing.JMenuItem quitItem;
    private javax.swing.JScrollPane jScrollPane1;
    private javax.swing.JButton bacard;
    private kiwi.ui.DateChooserField cardDdate;
    private javax.swing.JButton badir;
    private javax.swing.JButton bdel;
    private javax.swing.JCheckBox ccompleted;
    private javax.swing.JMenuBar jMenuBar1;
    private javax.swing.JTree directoryTree;
    private kiwi.ui.DateChooserField cardEdate;
    private javax.swing.JPanel jPanel9;
    private javax.swing.JPanel jPanel8;
    private javax.swing.JMenuItem logoutItem;
    private javax.swing.JButton bcopy;
    private javax.swing.JPanel jPanel7;
    private javax.swing.JPanel jPanel6;
    private javax.swing.JPanel jPanel5;
    private javax.swing.JPanel jPanel4;
    private javax.swing.JPanel jPanel3;
    private javax.swing.JPanel jPanel2;
    private javax.swing.JPanel jPanel1;
    private javax.swing.JPanel container;
    private javax.swing.JPanel note;
    private javax.swing.JSeparator jSeparator1;
    private javax.swing.JButton bpaste;
    private javax.swing.JTextField cardTitle;
    private javax.swing.JLabel jLabel6;
    private javax.swing.JLabel jLabel4;
    private javax.swing.JButton bcut;
    private javax.swing.JPanel empty;
    private javax.swing.JLabel jLabel2;
    private javax.swing.JLabel jLabel1;
    private javax.swing.JLabel directoryTitle;
    private javax.swing.JEditorPane dirAnnotation;
    private javax.swing.JSplitPane jSplitPane1;
    private javax.swing.JTextArea cardText;
    // End of variables declaration//GEN-END:variables
}
